unit TSP;

interface

type ARRNN = array[0..288,0..288] of integer;
type ARRN  = array[0..288] of integer;

procedure babtsp(
   n,inf        :integer;
   var w        :arrnn;
   var route    :arrn;
   var tweight  :integer);
PROCEDURE FITSP(N,S,INF:INTEGER;VAR W :ARRNN;VAR ROUTE  :ARRN;VAR TWEIGHT:INTEGER);
PROCEDURE TWOOPT(N :INTEGER;VAR W :ARRNN;VAR ROUTE  :ARRN; VAR TWEIGHT:INTEGER);
PROCEDURE THREEOPT(N :INTEGER;VAR W:ARRNN;VAR ROUTE  :ARRN;VAR TWEIGHT:INTEGER);
PROCEDURE anneal(x,y : ARRN;VAR W:ARRNN; VAR iorder: ARRN; ncity: integer);

implementation
VAR
   glinext,glinextp,gljdum: integer;
   glma: ARRAY [1..55] OF real;

procedure babtsp(
   n,inf        :integer;
   var w        :arrnn;
   var route    :arrn;
   var tweight  :integer);

var backptr,best,col,fwdptr,row  :arrn;
    i,index                      :integer;

procedure explore(
   edges,cost   :integer;
   var row,col  :arrn);
var avoid,c,colrowval,first,i,j,
          last,lowerbound,most,r,size  :integer;
    colred,newcol,newrow,rowred        :arrn;

function min(
   i,j  :integer) :integer;
begin
   if i <= j then min:=i
   else min :=j
end;

function reduce(
   var row,col,rowred,colred :arrn):integer;
var i,j,rvalue,temp  :integer;
begin
   rvalue:=0;
   for i:=1 to size do                              (* reduce rows *)
   begin
      temp:=inf;
      for j:=1 to size do
         temp:=min(temp,w[row[i],col[j]]);
      if temp > 0 then
      begin
         for j:=1 to size do
            if w[row[i],col[j]] < inf then
               w[row[i],col[j]]:=w[row[i],col[j]]-temp;
         rvalue:=rvalue+temp
      end;
      rowred[i]:=temp
   end;                                         (* reduce columns  *)
   for j:=1 to size do
   begin
      temp:=inf;
      for i:=1 to size do
         temp:=min(temp,w[row[i],col[j]]);
      if temp > 0 then
      begin
         for i:=1 to size do
            if w[row[i],col[j]] < inf then
               w[row[i],col[j]]:=w[row[i],col[j]]-temp;
         rvalue:=rvalue+temp
      end;
      colred[j]:=temp
   end;
   reduce:=rvalue
end; (* reduce *)

procedure bestedge(
   var r,c,most :integer);
var i,j,k,mincolelt,minrowelt,zeroes :integer;
begin
   most:=-inf;
   for i:=1 to size do
      for j:=1 to size do
      if w[row[i],col[j]] = 0 then
      begin
         minrowelt:=inf;
         zeroes:=0;
         for k:=1 to size do
            if w[row[i],col[k]] = 0 then zeroes:=zeroes+1
            else minrowelt:=min(minrowelt,w[row[i],col[k]]);
         if zeroes > 1 then minrowelt:=0;
         mincolelt:=inf;
         zeroes:=0;
         for k:=1 to size do
            if w[row[k],col[j]] = 0 then zeroes:=zeroes+1
            else mincolelt:=min(mincolelt,w[row[k],col[j]]);
         if zeroes > 1 then mincolelt:=0;
         if (minrowelt+mincolelt) > most then
         begin                     (* a better edge has been found *)
            most:=minrowelt+mincolelt;
            r:=i;                      (* row index of better edge *)
            c:=j                    (* column index of better edge *)
         end
   end
end; (* bestedge *)

begin (* explore *)
   size:=n-edges;           (* number of rows, cols left in matrix *)
   cost:=cost+reduce(row,col,rowred,colred);
   if cost < tweight then
      if edges = (n-2) then
      begin                           (* last two edges are forced *)
         for i:=1 to n do best[i]:=fwdptr[i];
         if w[row[1],col[1]] = inf then avoid:=1
         else avoid:=2;
         best[row[1]]:=col[3-avoid];
         best[row[2]]:=col[avoid];
         tweight:=cost
      end
      else
      begin
         bestedge(r,c,most);
         lowerbound:=cost+most;              (* record chosen edge *)
         fwdptr[row[r]]:=col[c];
         backptr[col[c]]:=row[r];
         last:=col[c];                           (* prevent cycles *)
         while fwdptr[last] <> 0 do last:=fwdptr[last];
         first:=row[r];
         while backptr[first] <> 0 do first:=backptr[first];
         colrowval:=w[last,first];
         w[last,first]:=inf;
         for i:=1 to r-1 do newrow[i]:=row[i];       (* remove row *)
         for i:=r to size-1 do newrow[i]:=row[i+1];
         for i:=1 to c-1 do newcol[i]:=col[i];       (* remove col *)
         for i:=c to size-1 do newcol[i]:=col[i+1];
         explore(edges+1,cost,newrow,newcol);
         w[last,first]:=colrowval;      (* restore previous values *)
         backptr[col[c]]:=0;
         fwdptr[row[r]]:=0;
         if lowerbound < tweight then
         begin
            w[row[r],col[c]]:=inf;   (* exclude edge alredy chosen *)
            explore(edges,cost,row,col);
            w[row[r],col[c]]:=0           (* restore excluded edge *)
         end
      end;
   for i:=1 to size do                          (* unreduce matrix *)
      for j:=1 to size do
         w[row[i],col[j]]:=w[row[i],col[j]]+rowred[i]+colred[j]
end; (* explore *)

begin  (* babtsp *)
   for i:=1 to n do
   begin
      row[i]:=i;
      col[i]:=i;
      fwdptr[i]:=0;
      backptr[i]:=0
   end;
   tweight:=inf;
   explore(0,0,row,col);
   index:=1;
   for i:=1 to n do
   begin
      route[i]:=index;
      index:=best[index]
   end
end; (* babtsp *)

PROCEDURE FITSP(N,S,INF:INTEGER;VAR W :ARRNN;VAR ROUTE  :ARRN;VAR TWEIGHT:INTEGER);

   VAR END1,END2,FARTHEST,I,INDEX,
          INSCOST,J,MAXDIST,NEWCOST,NEXTINDEX :INTEGER;
       CYCLE,DIST                             :ARRN;
BEGIN
   FOR I:=1 TO N DO CYCLE[I]:=0;
   CYCLE[S]:=S;
   FOR I:=1 TO N DO DIST[I]:=W[S,I];
   TWEIGHT:=0;
   FOR I:=1 TO N-1 DO BEGIN
      MAXDIST:=-INF;
      FOR J:=1 TO N DO
         IF CYCLE[J] = 0 THEN
            IF DIST[J] > MAXDIST THEN BEGIN
               MAXDIST:=DIST[J];  FARTHEST:=J
            END;
      INSCOST:=INF;  INDEX:=S;
      FOR J:=1 TO I DO  BEGIN
         NEXTINDEX:=CYCLE[INDEX];
         NEWCOST:=W[INDEX,FARTHEST]+W[FARTHEST,NEXTINDEX]-
                  W[INDEX,NEXTINDEX];
         IF NEWCOST < INSCOST THEN BEGIN
            INSCOST:=NEWCOST;
            END1:=INDEX;  END2:=NEXTINDEX
         END;
         INDEX:=NEXTINDEX
      END;  { FOR J }
      CYCLE[FARTHEST]:=END2;  CYCLE[END1]:=FARTHEST;
      TWEIGHT:=TWEIGHT+INSCOST;
      FOR J:=1 TO N DO
         IF CYCLE[J] = 0 THEN
            IF W[FARTHEST,J] < DIST[J] THEN DIST[J]:=W[FARTHEST,J]
   END;  { FOR I }
   INDEX:=S;
   FOR I:=1 TO N DO BEGIN
      ROUTE[I]:=INDEX;  INDEX:=CYCLE[INDEX]
   END
END;  { FITSP }

PROCEDURE TWOOPT(N :INTEGER;VAR W :ARRNN;VAR ROUTE  :ARRN; VAR TWEIGHT:INTEGER);

   VAR AHEAD,I,I1,I2,INDEX,J,J1,J2,
       LAST,LIMIT,MAX,MAX1,NEXT,S1,S2,T1,T2:INTEGER;
       PTR                                 :ARRN;
BEGIN
   FOR I:=1 TO N-1 DO PTR[ROUTE[I]]:=ROUTE[I+1];
   PTR[ROUTE[N]]:=ROUTE[1];
   REPEAT  { UNTIL MAX = 0 }
      MAX:=0;  I1:=1;
      FOR I:=1 TO N-2 DO BEGIN
         IF I = 1 THEN LIMIT:=N-1
         ELSE LIMIT:=N;
         I2:=PTR[I1];  J1:=PTR[I2];
         FOR J:=I+2 TO LIMIT DO BEGIN
            J2:=PTR[J1];
            MAX1:=W[I1,I2]+W[J1,J2]-(W[I1,J1]+W[I2,J2]);
            IF MAX1 > MAX THEN BEGIN   { BETTER PAIR HAS BEEN FOUND }
               S1:=I1;  S2:=I2;
               T1:=J1;  T2:=J2;
               MAX:=MAX1
            END;
            J1:=J2
         END;  { FOR J }
         I1:=I2
      END;  { FOR I }
      IF MAX > 0 THEN BEGIN                    { SWAP PAIR OF EDGES }
         PTR[S1]:=T1;
         NEXT:=S2;  LAST:=T2;
         REPEAT
            AHEAD:=PTR[NEXT];  PTR[NEXT]:=LAST;
            LAST:=NEXT;  NEXT:=AHEAD
         UNTIL NEXT = T2;
         TWEIGHT:=TWEIGHT-MAX
      END  { IF MAX > 0 }
   UNTIL MAX = 0;
   INDEX:=1;
   FOR I:=1 TO N DO BEGIN
      ROUTE[I]:=INDEX;  INDEX:=PTR[INDEX]
   END
END;  { TWOOPT }

PROCEDURE THREEOPT(N :INTEGER;VAR W:ARRNN;VAR ROUTE  :ARRN;VAR TWEIGHT:INTEGER);

   TYPE SWAPTYPE  =(ASYMMETRIC,SYMMETRIC);
        SWAPRECORD=RECORD
                      X1,X2,Y1,Y2,Z1,Z2,GAIN:INTEGER;
                      CHOICE                :SWAPTYPE
                   END;
   VAR BESTSWAP,SWAP:SWAPRECORD;
       I,INDEX,J,K  :INTEGER;
       PTR          :ARRN;

   PROCEDURE SWAPCHECK(VAR SWAP:SWAPRECORD);
      VAR DELWEIGHT,MAX:INTEGER;
   BEGIN
      WITH SWAP DO BEGIN
         GAIN:=0;
         DELWEIGHT:=W[X1,X2]+W[Y1,Y2]+W[Z1,Z2];
         MAX:=DELWEIGHT-(W[Y1,X1]+W[Z1,X2]+W[Z2,Y2]);
         IF MAX > GAIN THEN BEGIN
            GAIN:=MAX;  CHOICE:=ASYMMETRIC
         END;
         MAX:=DELWEIGHT-(W[X1,Y2]+W[Z1,X2]+W[Y1,Z2]);
         IF MAX > GAIN THEN BEGIN
            GAIN:=MAX;  CHOICE :=SYMMETRIC
         END
      END  { WITH SWAP }
   END;  { SWAPCHECK }

   PROCEDURE REVERSE(START,FINISH:INTEGER);
      VAR AHEAD,LAST,NEXT:INTEGER;
   BEGIN
      IF START <> FINISH THEN BEGIN
         LAST:=START;  NEXT:=PTR[LAST];
         REPEAT
            AHEAD:=PTR[NEXT];  PTR[NEXT]:=LAST;
            LAST:=NEXT;  NEXT:=AHEAD;
         UNTIL LAST = FINISH
      END  { IF START <> FINISH }
   END;  { RESERVE }

BEGIN                                                   { MAIN BODY }
   FOR I:=1 TO N-1 DO PTR[ROUTE[I]]:=ROUTE[I+1];
   PTR[ROUTE[N]]:=ROUTE[1];
   REPEAT  { UNTIL BESTSWAP.GAIN = 0 }
      BESTSWAP.GAIN:=0;
      SWAP.X1:=1;
      FOR I:=1 TO N DO BEGIN
         SWAP.X2:=PTR[SWAP.X1];  SWAP.Y1:=SWAP.X2;
         FOR J:=2 TO N-3 DO BEGIN
            SWAP.Y2:=PTR[SWAP.Y1];  SWAP.Z1:=PTR[SWAP.Y2];
            FOR K:=J+2 TO N-1 DO BEGIN
               SWAP.Z2:=PTR[SWAP.Z1];
               SWAPCHECK(SWAP);
               IF SWAP.GAIN > BESTSWAP.GAIN THEN BESTSWAP:=SWAP;
               SWAP.Z1:=SWAP.Z2
            END;  { FOR K }
            SWAP.Y1:=SWAP.Y2
         END;  { FOR J }
         SWAP.X1:=SWAP.X2
      END;  { FOR I }
      IF BESTSWAP.GAIN > 0 THEN
         WITH BESTSWAP DO BEGIN
            IF CHOICE = ASYMMETRIC THEN BEGIN
               REVERSE(Z2,X1);
               PTR[Y1]:=X1;  PTR[Z2]:=Y2
            END
            ELSE BEGIN
               PTR[X1]:=Y2;  PTR[Y1]:=Z2
            END;
            PTR[Z1]:=X2;
            TWEIGHT:=TWEIGHT-GAIN
         END  { WITH BESTSWAP }
   UNTIL BESTSWAP.GAIN = 0;
   INDEX:=1;
   FOR I:=1 TO N DO BEGIN
      ROUTE[I]:=INDEX;  INDEX:=PTR[INDEX]
   END
END;  { THREEOPT }

// SIMULATED ANNEALING
FUNCTION irbit1(VAR iseed: integer): integer;
(* This routine runs much faster if you can perform bitwise logical operations
on integers.  For example, here is a TURBO Pascal version:
CONST
   ib1=1; ib3=4; ib5=16; ib14=8192;
VAR
   newbit: boolean;
BEGIN
   newbit := (iseed AND ib14) <> 0;
   IF ((iseed AND ib5) <> 0) THEN newbit :=  NOT newbit;
   IF ((iseed AND ib3) <> 0) THEN newbit :=  NOT newbit;
   IF ((iseed AND ib1) <> 0) THEN newbit :=  NOT newbit;
   iseed := iseed SHL 1;
   IF (newbit) THEN BEGIN
      irbit1 := 1; iseed := iseed OR ib1
   END ELSE BEGIN
      irbit1 := 0; iseed := iseed AND (NOT ib1)
   END
END; *)
(* Here is the slower version for other Pascal systems: *)
CONST
   ib1=1;
   ib3=4;
   ib5=16;
   ib14=8192;    (* Values chosen not to overflow 2-byte integers *)
VAR
   mask: integer;
   newbit: boolean;
FUNCTION iand(i1,i2: integer): integer;
   VAR
      i: integer;
   BEGIN
      IF ((i1=0) OR (i2=0)) THEN iand := 0
      ELSE BEGIN
         i := ord(odd(i1) AND odd(i2));
         i1 := i1 DIV 2; i2 := i2 DIV 2;
         iand := 2*iand(i1,i2) + i
      END
   END;
FUNCTION inot(ib: integer): integer;
   BEGIN inot := maxint-ib END;
FUNCTION ior(i1,i2: integer): integer;
   VAR
      i: integer;
   BEGIN
      IF ((i1=0) AND (i2=0)) THEN ior := 0
      ELSE BEGIN
         i := ord(odd(i1) OR odd(i2));
         i1 := i1 DIV 2; i2 := i2 DIV 2;
         ior := 2*ior(i1,i2) + i
      END
   END;
BEGIN
   mask := maxint DIV 2;
   newbit := iand(iseed,ib14) <> 0;
   IF (iand(iseed,ib5) <> 0) THEN newbit :=  NOT newbit;
   IF (iand(iseed,ib3) <> 0) THEN newbit :=  NOT newbit;
   IF (iand(iseed,ib1) <> 0) THEN newbit :=  NOT newbit;
   irbit1 := 0;
   iseed := iand(2*iand(mask,iseed),inot(ib1));
   IF (newbit) THEN BEGIN
      irbit1 := 1;
      iseed := ior(iseed,ib1);
   END
END;

FUNCTION ran3(VAR idum: integer): real;
(* Programs using RAN3 must declare the following variables
VAR
   glinext,glinextp: integer;
   glma: ARRAY [1..55] OF real;
in the main routine. Machines with 4-byte integers can use the integer
implementation of this routine, substituting glma of type integer, the
commented CONST and VAR declarations, and the MOD function in the third
line after the BEGIN. *)
(* CONST
   mbig=1000000000;
   mseed=161803398;
   mz=0;
   fac=1.0e-9;
VAR
   i,ii,k,mj,mk: integer; *)
CONST
   mbig=4.0e6;
   mseed=1618033.0;
   mz=0.0;
   fac=2.5e-7; (* 1/mbig *)
VAR
   i,ii,k: integer;
   mj,mk: real;
BEGIN
   IF (idum < 0) THEN BEGIN
      mj := mseed+idum;
      (* The following IF block is mj := mj MOD mbig; for real variables. *)
      IF mj>=0.0 THEN mj := mj-mbig*trunc(mj/mbig)
         ELSE mj := mbig-abs(mj)+mbig*trunc(abs(mj)/mbig);
      glma[55] := mj;
      mk := 1;
      FOR i := 1 TO 54 DO BEGIN
         ii := 21*i MOD 55;
         glma[ii] := mk;
         mk := mj-mk;
         IF (mk < mz) THEN mk := mk+mbig;
         mj := glma[ii]
      END;
      FOR k := 1 TO 4 DO BEGIN
         FOR i := 1 TO 55 DO BEGIN
            glma[i] := glma[i]-glma[1+((i+30) MOD 55)];
            IF (glma[i] < mz) THEN glma[i] := glma[i]+mbig
         END
      END;
      glinext := 0;
      glinextp := 31;
      idum := 1
   END;
   glinext := glinext+1;
   IF (glinext = 56) THEN glinext := 1;
   glinextp := glinextp+1;
   IF (glinextp = 56) THEN glinextp := 1;
   mj := glma[glinext]-glma[glinextp];
   IF (mj < mz) THEN mj := mj+mbig;
   glma[glinext] := mj;
   ran3 := mj*fac
END;




PROCEDURE anneal(x,y : ARRN;VAR W:ARRNN; VAR iorder: ARRN; ncity: integer);
(* Programs using routine ANNEAL must define types
   ARRN : ARRAY [1..ncity] OF real;
   ARRN : ARRAY [1..ncity] OF integer;
in the main routine. *)
LABEL 10,20,99;
CONST
   tfactr = 0.9;
TYPE
   nsix = ARRAY [1..6] OF integer;
VAR
   ans : boolean;
   path,de,t : real;
   nover,nlimit,i1,i2,idum,iseed: integer;
   i,j,k,nsucc,nn,idec : integer;
   n : nsix;
 
FUNCTION alen(x1,x2,y1,y2: integer): integer;
BEGIN
   alen := w[x1,x2];
END;

PROCEDURE revcst(x,y: ARRN; iorder: ARRN; ncity: integer;
      VAR n: nsix; VAR de: real);
VAR
   xx,yy : ARRAY [1..6] OF integer;
   j,ii : integer;
BEGIN
   n[3] := 1 + ((n[1]+ncity-2) MOD ncity);
   n[4] := 1 + (n[2] MOD ncity);
   FOR j := 1 TO 4 DO BEGIN
      ii := iorder[n[j]];
      xx[j] := x[ii];
      yy[j] := y[ii]
   END;
   de := -alen(xx[1],xx[3],yy[1],yy[3])-alen(xx[2],xx[4],yy[2],yy[4])
      +alen(xx[1],xx[4],yy[1],yy[4])+alen(xx[2],xx[3],yy[2],yy[3])
END;

PROCEDURE reverse(VAR iorder: ARRN; ncity: integer; n: nsix);
VAR
   nn,j,k,l,itmp : integer;
BEGIN
   nn := (1+((n[2]-n[1]+ncity) MOD ncity)) DIV 2;
   FOR j := 1 TO nn DO BEGIN
      k := 1 + ((n[1]+j-2) MOD ncity);
      l := 1 + ((n[2]-j+ncity) MOD ncity);
      itmp := iorder[k];
      iorder[k] := iorder[l];
      iorder[l] := itmp
   END
END;

PROCEDURE trncst(x,y: ARRN; iorder: ARRN; ncity: integer;
      VAR n: nsix; VAR de: real);
VAR
   xx,yy : ARRAY [1..6] OF integer;
   j,ii : integer;
BEGIN
   n[4] := 1 + (n[3] MOD ncity);
   n[5] := 1 + ((n[1]+ncity-2) MOD ncity);
   n[6] := 1 + (n[2] MOD ncity);
   FOR j := 1 TO 6 DO BEGIN
      ii := iorder[n[j]];
      xx[j] := x[ii];
      yy[j] := y[ii]
   END;
   de := -alen(xx[2],xx[6],yy[2],yy[6])-alen(xx[1],xx[5],yy[1],yy[5])
      -alen(xx[3],xx[4],yy[3],yy[4])+alen(xx[1],xx[3],yy[1],yy[3])
      +alen(xx[2],xx[4],yy[2],yy[4])+alen(xx[5],xx[6],yy[5],yy[6])
END;
 
PROCEDURE trnspt(VAR iorder: ARRN; ncity: integer; n: nsix);
CONST
   maxcity=1000;
VAR
   jorder : ARRAY [1..maxcity] OF integer;
   m1,m2,m3,nn,j,jj : integer;
BEGIN
   m1 := 1 + ((n[2]-n[1]+ncity) MOD ncity);
   m2 := 1 + ((n[5]-n[4]+ncity) MOD ncity);
   m3 := 1 + ((n[3]-n[6]+ncity) MOD ncity);
   nn := 1;
   FOR j := 1 TO m1 DO BEGIN
      jj := 1 + ((j+n[1]-2) MOD ncity);
      jorder[nn] := iorder[jj];
      nn := nn+1
   END;
   IF (m2>0) THEN BEGIN
      FOR j := 1 TO m2 DO BEGIN
         jj := 1+((j+n[4]-2) MOD ncity);
         jorder[nn] := iorder[jj];
         nn := nn+1
      END
   END;
   IF (m3>0) THEN BEGIN
      FOR j := 1 TO m3 DO BEGIN
         jj := 1 + ((j+n[6]-2) MOD ncity);
         jorder[nn] := iorder[jj];
         nn := nn+1
      END
   END;
   FOR j := 1 TO ncity DO BEGIN
      iorder[j] := jorder[j] 
   END
END;
 
PROCEDURE metrop(de,t: real; VAR ans: boolean);
(* Programs using routine METROP must declare the variable 
VAR
   gljdum : integer;
and initialize its value to
   gljdum := 1;
in the main routine. *)
BEGIN
   ans := (de<0.0) OR (ran3(gljdum)<exp(-de/t))
END;
 
BEGIN
   nover := 100*ncity;
   nlimit := 10*ncity;
   path := 0.0;
   t := 0.5;
   FOR i := 1 TO (ncity-1) DO BEGIN
      i1 := iorder[i];
      i2 := iorder[i+1];
      path := path+alen(x[i1],x[i2],y[i1],y[i2])
   END;
   i1 := iorder[ncity];
   i2 := iorder[1];
   path := path+alen(x[i1],x[i2],y[i1],y[i2]);
   idum := -1;
   iseed := 111;
   FOR j := 1 TO 100 DO BEGIN
      nsucc := 0;
      FOR k := 1 TO nover DO BEGIN
10:      n[1] := 1+trunc(ncity*ran3(idum));
         n[2] := 1+trunc((ncity-1)*ran3(idum));
         IF (n[2]>=n[1]) THEN n[2] := n[2]+1;
         nn := 1+((n[1]-n[2]+ncity-1) MOD ncity);
         IF (nn<3) THEN goto 10;
         idec := irbit1(iseed);
         IF (idec=0) THEN BEGIN
            n[3] := n[2]+trunc(abs(nn-2)*ran3(idum))+1;
            n[3] := 1+((n[3]-1) MOD ncity);
            trncst(x,y,iorder,ncity,n,de);
            metrop(de,t,ans);
            IF ans THEN BEGIN
               nsucc := nsucc+1;
               path := path+de;
               trnspt(iorder,ncity,n)
            END
         END ELSE BEGIN
            revcst(x,y,iorder,ncity,n,de);
            metrop(de,t,ans);
            IF ans THEN BEGIN
               nsucc := nsucc+1;
               path := path+de;
               reverse(iorder,ncity,n)
            END
         END;
         IF (nsucc>=nlimit) THEN goto 20
      END;
20:
//writeln;
//      writeln('T =',t:10:6,'    Path Length =',path:12:6);
//      writeln('Successful Moves: ',nsucc:6);
      t := t*tfactr;
      IF (nsucc=0) THEN goto 99
   END;
99:
END;


BEGIN
gljdum:=1;
end.
 
